"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
const lang_runner_nodejs_1 = require("../lib/lang-runner-nodejs");
const lang_runner_java_1 = require("../lib/lang-runner-java");
class LocalRun {
    constructor(lang = 'auto', runnerOpts = {}, runners) {
        this.lang = lang;
        runners =
            runners !== null && runners !== void 0 ? runners : new Map([
                ['nodejs', lang_runner_nodejs_1.default],
                ['java', lang_runner_java_1.default],
            ]);
        this.runners = new Map();
        for (const [l, r] of runners) {
            this.runners.set(l, new r(runnerOpts));
        }
    }
    async exec() {
        let runner;
        switch (this.lang) {
            case 'auto':
                runner = await this.autoDetect();
                break;
            case 'javascript':
            case 'typescript':
                runner = this.runners.get('nodejs');
                break;
            default:
                runner = this.runners.get(this.lang);
        }
        if (!runner) {
            throw new Error(`Could not determine local function invoker for language: ${this.lang}`);
        }
        await runner.build();
        await runner.start();
        return;
    }
    async autoDetect() {
        const detectionPromises = [];
        for (const [k, runner] of this.runners.entries()) {
            detectionPromises.push(runner.detect().then((detected) => (detected ? runner : undefined)));
        }
        const detectionResults = await Promise.all(detectionPromises);
        return detectionResults.find((r) => r);
    }
}
exports.default = LocalRun;
